@@ -6,10 +6,12 @@ from .pages import (  # noqa
     PdfPage,
     ListPage,
     CsvListPage,
+    ExcelListPage,
     HtmlListPage,
     JsonListPage,
     XmlListPage,
-    page_to_items,
+    MissingSourceError,
 )
 from .selectors import SelectorError, Selector, XPath, SimilarLink, CSS  # noqa
-from .core import Workflow, Source, URL, NullSource  # noqa
+from .sources import Source, URL, NullSource  # noqa
+from .workflow import page_to_items, Workflow  # noqa
