@@ -1,21 +1,34 @@
 import dataclasses
+import datetime
+import glob
 import importlib
+import logging
+import os
+import sys
 import typing
 import lxml.html
 import click
 from scrapelib import Scraper
 from .utils import _display
-from .core import URL
+from .sources import URL
+from .workflow import Workflow
 
 try:
     from attr import has as attr_has
     from attr import fields as attr_fields
-except ImportError:
+except ImportError:  # pragma: no cover
     attr_has = lambda x: False  # noqa
     attr_fields = lambda x: []  # noqa
 
 
-VERSION = "0.4.1"
+VERSION = "0.5.0"
+
+
+def configure_logging(level):
+    # replace scrapelib's logging
+    logging.getLogger("scrapelib").setLevel(logging.ERROR)
+    logging.getLogger("urllib3").setLevel(logging.WARNING)
+    logging.basicConfig(level=level)
 
 
 def get_class(dotted_name: str):
@@ -45,8 +58,8 @@ def shell(url: str, user_agent: str, verb: str) -> None:
     """
     try:
         from IPython import embed
-    except ImportError:
-        print("shell command requires IPython")
+    except ImportError:  # pragma: no cover
+        click.secho("shell command requires IPython", fg="red")
         return
 
     # import selectors so they can be used without import
@@ -79,20 +92,20 @@ def _get_fake_input(Cls, data, interactive):
         return getattr(Cls, "example_input")
 
     if input_type:
-        print(f"{Cls.__name__} expects input ({input_type.__name__}): ")
+        click.secho(f"{Cls.__name__} expects input ({input_type.__name__}): ")
         if dataclasses.is_dataclass(input_type):
             fields = dataclasses.fields(input_type)
         elif attr_has(input_type):
             fields = attr_fields(input_type)
         for field in fields:
             if field.name in fake_input:
-                print(f"  {field.name}: {fake_input[field.name]}")
+                click.secho(f"  {field.name}: {fake_input[field.name]}")
             elif interactive:
                 fake_input[field.name] = click.prompt("  " + field.name)
             else:
                 dummy_val = f"~{field.name}"
                 fake_input[field.name] = dummy_val
-                print(f"  {field.name}: {dummy_val}")
+                click.secho(f"  {field.name}: {dummy_val}")
         return input_type(**fake_input)
     else:
         return fake_input
@@ -100,24 +113,46 @@ def _get_fake_input(Cls, data, interactive):
 
 @cli.command()
 @click.argument("class_name")
-@click.option("-i", "--interactive", help="Interactively prompt for missing data.")
+@click.option(
+    "--interactive/--no-interactive",
+    default=False,
+    help="Interactively prompt for missing data.",
+)
 @click.option(
     "-d", "--data", multiple=True, help="Provide input data in name=value pairs."
 )
 @click.option("-s", "--source", help="Provide (or override) source URL")
+@click.option(
+    "--pagination/--no-pagination",
+    default=True,
+    help="Determine whether or not pagination should be followed or one page is "
+    "enough for testing",
+)
 def test(
-    class_name: str, interactive: bool, data: typing.List[str], source: str
+    class_name: str,
+    interactive: bool,
+    data: typing.List[str],
+    source: str,
+    pagination: bool,
 ) -> None:
     """
-    This command allows you to scrape a single page and see the output immediately.  This eases the common cycle of making modifications to a scraper, running a scrape (possibly with long-running but irrelevant portions commented out), and comparing output to what is expected.
-    ``test`` can also be useful for debugging existing scrapers, you can see exactly what a single step of the scrape is providing, to help narrow down where erroneous data is coming from.
+    This command allows you to scrape a single page and see the output immediately.
+    This eases the common cycle of making modifications to a scraper, running a scrape
+    (possibly with long-running but irrelevant portions commented out), and comparing
+    output to what is expected.
+
+    ``test`` can also be useful for debugging existing scrapers, you can see exactly
+    what a single step of the scrape is providing, to help narrow down where
+    erroneous data is coming from.
 
     Example::
 
     $ spatula test path.to.ClassName --source https://example.com
 
-    This will run the scraper defined at :py:class:`path.to.ClassName` against the provided URL.
+    This will run the scraper defined at :py:class:`path.to.ClassName` against the
+    provided URL.
     """
+    configure_logging(logging.DEBUG)
     Cls = get_class(class_name)
     s = Scraper()
 
@@ -131,6 +166,7 @@ def test(
 
     # we need to do the request-response-next-page loop at least once
     once = True
+    num_items = 0
     while source or once:
         once = False
         page = Cls(fake_input, source=source)
@@ -141,17 +177,28 @@ def test(
         result = page.process_page()
 
         if isinstance(result, typing.Generator):
-            for i, item in enumerate(result):
-                print(f"{i}:", _display(item))
+            for item in result:
+                # use this count instead of enumerate to handle pagination
+                num_items += 1
+                click.echo(click.style(f"{num_items}: ", fg="green") + _display(item))
         else:
-            print(_display(result))
+            click.secho(_display(result))
 
         # will be None in most cases, existing the loop, otherwise we restart
         source = page.get_next_source()
         if source:
-            click.secho(
-                f"paginating for {page.__class__.__name__}: {source}", fg="blue"
-            )
+            if pagination:
+                click.secho(
+                    f"paginating for {page.__class__.__name__} source={source}",
+                    fg="blue",
+                )
+            else:
+                click.secho(
+                    "pagination disabled: would paginate for "
+                    f"{page.__class__.__name__} source={source}",
+                    fg="yellow",
+                )
+                break
 
 
 @cli.command()
@@ -163,9 +210,32 @@ def scrape(workflow_name: str, output_dir: str) -> None:
     """
     Run full workflow, and output data to disk.
     """
-    workflow = get_class(workflow_name)
-    workflow.execute(output_dir=output_dir)
+    configure_logging(logging.INFO)
+    workflow_or_page = get_class(workflow_name)
+    if isinstance(workflow_or_page, Workflow):
+        workflow = workflow_or_page
+    else:
+        workflow = Workflow(workflow_or_page)
+    if not output_dir:
+        dirn = 1
+        today = datetime.date.today().strftime("%Y-%m-%d")
+        while True:
+            try:
+                output_dir = f"_scrapes/{today}/{dirn:03d}"
+                os.makedirs(output_dir)
+                break
+            except FileExistsError:
+                dirn += 1
+    else:
+        try:
+            os.makedirs(output_dir)
+        except FileExistsError:
+            if len(glob.glob(output_dir + "/*")):
+                click.secho(f"{output_dir} exists and is not empty", fg="red")
+                sys.exit(1)
+    count = workflow.execute(output_dir=output_dir)
+    click.secho(f"success: wrote {count} objects to {output_dir}", fg="green")
 
 
-if __name__ == "__main__":
+if __name__ == "__main__":  # pragma: no cover
     cli()
