@@ -3,39 +3,20 @@ import csv
 import tempfile
 import subprocess
 import logging
-import typing
-import lxml.html
 import scrapelib
-from .core import URL, HandledError
+import lxml.html
+from openpyxl import load_workbook
+from .sources import URL
 
 
-def page_to_items(scraper, page):
-    # fetch data for a page, and then call the process_page entrypoint
-    page._fetch_data(scraper)
-    result = page.process_page()
+class MissingSourceError(Exception):
+    def __init__(self, msg):
+        super().__init__(msg)
 
-    # if we got back a generator, we need to process each result
-    if isinstance(result, typing.Generator):
-        # each item yielded might be a Page or an end-result
-        for item in result:
-            if isinstance(item, Page):
-                yield from page_to_items(scraper, item)
-            else:
-                yield item
 
-        # after handling a list, check for pagination
-        next_source = page.get_next_source()
-        if next_source:
-            # instantiate the same class with same input, but increment the source
-            yield from page_to_items(
-                scraper, type(page)(page.input, source=next_source)
-            )
-    elif isinstance(result, Page):
-        # single Page result, recurse deeper
-        yield from page_to_items(scraper, result)
-    else:
-        # end-result, just return as-is
-        yield result
+class HandledError(Exception):
+    def __init__(self, exc):
+        super().__init__(exc)
 
 
 class Page:
@@ -44,8 +25,8 @@ class Page:
 
     def _fetch_data(self, scraper: scrapelib.Scraper) -> None:
         """
-        ensure that the page has all of its data, this is guaranteed to be called exactly once
-        before process_page is invoked
+        ensure that the page has all of its data, this is guaranteed to be called
+        exactly once before process_page is invoked
         """
         # process dependencies first
         for val, dep in self.dependencies.items():
@@ -60,7 +41,7 @@ class Page:
             elif hasattr(self.input, "url"):
                 self.source = URL(self.input.url)
             else:
-                raise Exception(
+                raise MissingSourceError(
                     f"{self.__class__.__name__} has no source or get_source_from_input"
                 )
         if isinstance(self.source, str):
@@ -69,7 +50,7 @@ class Page:
         try:
             self.response = self.source.get_response(scraper)
         except scrapelib.HTTPError as e:
-            self.handle_error_response(e)
+            self.process_error_response(e)
             raise HandledError(e)
         else:
             self.postprocess_response()
@@ -79,7 +60,18 @@ class Page:
         # allow possibility to override default source, useful during dev
         if source:
             self.source = source
-        self.logger = logging.getLogger(self.__class__.__name__)
+        self.logger = logging.getLogger(
+            self.__class__.__module__ + "." + self.__class__.__name__
+        )
+
+    def __str__(self):
+        s = f"{self.__class__.__name__}("
+        if self.input:
+            s += f"input={self.input} "
+        if self.source:
+            s += f"source={self.source}"
+        s += ")"
+        return s
 
     def postprocess_response(self) -> None:
         """
@@ -95,7 +87,7 @@ class Page:
 
         This is called after source.get_response if an exception is raised.
         """
-        pass
+        raise exception
 
     def process_page(self):
         """
@@ -143,7 +135,7 @@ class JsonPage(Page):
         self.data = self.response.json()
 
 
-class PdfPage(Page):
+class PdfPage(Page):  # pragma: no cover
     preserve_layout = False
 
     def postprocess_response(self) -> None:
@@ -172,10 +164,20 @@ class PdfPage(Page):
 
 class ListPage(Page):
     class SkipItem(Exception):
-        pass
+        def __init__(self, msg):
+            super().__init__(msg)
 
-    def skip(self) -> None:
-        raise self.SkipItem()
+    def skip(self, msg: str = "") -> None:
+        raise self.SkipItem(msg)
+
+    def _process_or_skip_loop(self, iterable):
+        for item in iterable:
+            try:
+                item = self.process_item(item)
+            except self.SkipItem as e:
+                self.logger.debug(f"SkipItem: {e}")
+                continue
+            yield item
 
     def process_item(self, item):
         return item
@@ -186,12 +188,20 @@ class CsvListPage(ListPage):
         self.reader = csv.DictReader(io.StringIO(self.response.text))
 
     def process_page(self):
-        for item in self.reader:
-            try:
-                item = self.process_item(item)
-            except self.SkipItem:
-                continue
-            yield item
+        yield from self._process_or_skip_loop(self.reader)
+
+    def process_item(self, item):
+        return item
+
+
+class ExcelListPage(ListPage):
+    def postprocess_response(self) -> None:
+        workbook = load_workbook(io.BytesIO(self.response.content))
+        # TODO: allow selecting this with a class property
+        self.worksheet = workbook.active
+
+    def process_page(self):
+        yield from self._process_or_skip_loop(self.worksheet.values)
 
     def process_item(self, item):
         return item
@@ -199,7 +209,8 @@ class CsvListPage(ListPage):
 
 class LxmlListPage(ListPage):
     """
-    Base class for XML and HTML subclasses below, only difference is which parser is used.
+    Base class for XML and HTML subclasses below, only difference is which
+    parser is used.
 
     Simplification for pages that get a list of items and process them.
 
@@ -213,12 +224,7 @@ class LxmlListPage(ListPage):
         if not self.selector:
             raise NotImplementedError("must either provide selector or override scrape")
         items = self.selector.match(self.root)
-        for item in items:
-            try:
-                item = self.process_item(item)
-            except self.SkipItem:
-                continue
-            yield item
+        yield from self._process_or_skip_loop(items)
 
 
 class HtmlListPage(LxmlListPage, HtmlPage):
@@ -231,9 +237,4 @@ class XmlListPage(LxmlListPage, XmlPage):
 
 class JsonListPage(ListPage, JsonPage):
     def process_page(self):
-        for item in self.data:
-            try:
-                item = self.process_item(item)
-            except self.SkipItem:
-                continue
-            yield item
+        yield from self._process_or_skip_loop(self.data)
